==> oscars_etl-0.1.1-py3-none-any.whl: oscars_etl/stats.py <==
import logging

LOGGER = logging.getLogger(__name__)


class Stat(object):

    def __init__(self, username, timestamp, collection,
                 product_id, product_date, product_size):

        self.username = username
        self.timestamp = timestamp
        self.collection = collection
        self.product_id = product_id
        self.product_date = product_date
        self.product_size = product_size

    @staticmethod
    def from_es_record(es_record):
        request = es_record['_source']['request']
        return Stat(es_record['_source']['auth'],
                    Stat._str_to_datetime(es_record['_source']['@timestamp']),
                    Stat._get_collection(request),
                    Stat._get_product_id(request),
                    Stat._get_product_date(request),
                    es_record['_source']['bytes'])

    @staticmethod
    def _clean_request(request):
        import re
        # remove download prefix
        request = re.sub('.*download/{1,}', '', request)
        # replace 2 or more slashes by one
        return re.sub('/{2,}', '/', request)

    @staticmethod
    def _str_to_datetime(timestamp):
        import dateutil.parser
        return dateutil.parser.isoparse(timestamp)

    @staticmethod
    def _get_collection(request):
        request = Stat._clean_request(request)
        url_parts = request.split('/')
        if url_parts[0] == 'Sentinel2' or url_parts[0] == 'PROBAV':
            return url_parts[0] + '_' + url_parts[1]
        else:
            return url_parts[0]

    @staticmethod
    def _get_product_id(request):
        collection = Stat._get_collection(request)
        request = Stat._clean_request(request)
        import re
        if re.match('Sentinel2_*', collection):
            url_parts = request.split('/')
            return url_parts[5]
        elif re.match('CGS_S1_SLC_L1', collection):
            url_parts = request.split('/')
            return url_parts[6]
        elif re.match('CGS_S1_GRD_L1', collection):
            url_parts = request.split('/')
            return url_parts[7]
        elif re.match('PROBAV_*', collection) \
                or re.match('CGS_S1_GRD_SIGMA0*', collection) \
                or re.match('S1_*', collection):
            url_parts = request.split('/')
            return url_parts[4]

        else:
            raise Exception('Unsupported collection {}'.format(collection))

    @staticmethod
    def _get_product_date(request):
        collection = Stat._get_collection(request)
        request = Stat._clean_request(request)
        import re
        from datetime import datetime
        if re.match('Sentinel2_*', collection):
            url_parts = request.split('/')
            return datetime(int(url_parts[2]), int(url_parts[3]), int(url_parts[4]))
        elif re.match('CGS_S1_SLC_L1', collection):
            url_parts = request.split('/')
            return datetime(int(url_parts[3]), int(url_parts[4]), int(url_parts[5]))
        elif re.match('CGS_S1_GRD_L1', collection):
            url_parts = request.split('/')
            return datetime(int(url_parts[4]), int(url_parts[5]), int(url_parts[6]))
        elif re.match('S1_*', collection) \
                or re.match('CGS_S1_GRD_SIGMA0*', collection):
            url_parts = request.split('/')
            return datetime(int(url_parts[1]), int(url_parts[2]), int(url_parts[3]))
        elif re.match('PROBAV_*', collection):
            url_parts = request.split('/')
            return datetime.strptime(url_parts[3], '%Y%m%d')
        else:
            raise Exception('Unsupported collection {}'.format(collection))


class CsvExporter(object):

    def __init__(self, csv_file):
        self.csv_file = csv_file

    def export(self, stats):
        with open(self.csv_file, 'a+') as stats_file:
            import csv
            stats_writer = csv.writer(stats_file, delimiter=',')
            for stat in stats:
                stats_writer.writerow([stat.username, stat.timestamp, stat.collection,
                                       stat.product_date, stat.product_id, stat.product_size])


class DatabaseExporter(object):

    def __init__(self, db_host, db_name, db_username, db_password):

        self.db_host = db_host
        self.db_name = db_name
        self.db_username = db_username
        self.db_password = db_password

        try:
            import psycopg2
            self.conn = psycopg2.connect("host='{}' dbname='{}' user='{}' password='{}'"
                                    .format(self.db_host, self.db_name, self.db_username, self.db_password))
        except Exception as ex:
            LOGGER.error('Could not connect to database: {}'.format(ex))
            raise

    def get_latest_stat(self):

        from psycopg2 import sql

        cursor = None
        try:
            cursor = self.conn.cursor()
            cursor.execute(
                sql.SQL("select max(download_timestamp) from oscars_stats"))
            results = cursor.fetchone()
            return results[0]
        finally:
            if cursor is not None:
                cursor.close()

    def export(self, stats):

        from psycopg2 import sql

        cursor = None
        try:
            cursor = self.conn.cursor()
            for stat in stats:
                cursor.execute(
                    sql.SQL(
                        """INSERT INTO oscars_stats (username, download_timestamp, collection, product_date, product_id, product_size) 
                            VALUES (%s, %s, %s, %s, %s, %s)"""),
                    [stat.username, stat.timestamp, stat.collection, stat.product_date, stat.product_id, stat.product_size])

            self.conn.commit()
        finally:
            if cursor is not None:
                cursor.close()

    def close(self):
        if self.conn is not None:
            self.conn.close()


class StatsExporter(object):

    def __init__(self, es_hosts):
        self.es_hosts = es_hosts

    def export_csv(self, start, end, csv_file):

        exporter = CsvExporter(csv_file)
        self._export_stats(start, end, exporter)

    def export_db(self, start, end,
                  db_host, db_name, db_username, db_password):

        exporter = DatabaseExporter(db_host, db_name, db_username, db_password)

        if start == None:
            # try to retrieve most recent download from DB
            start = exporter.get_latest_stat()

        self._export_stats(start, end, exporter)

    @staticmethod
    def _datetime2str(timestamp):
        import dateutil.parser
        return timestamp.strftime('%Y-%m-%dT%H:%M:%S.000Z')

    def _export_stats(self, start, end, exporter):

        from elasticsearch7 import Elasticsearch
        es = Elasticsearch(hosts=self.es_hosts)

        es_scroll_timeout = "1m"

        LOGGER.info("Start searching download statistics")

        query = {
            "query": {
                "bool": {
                    "must": [
                        {"term": {"host.name": "oscars-download01.vgt.vito.be"}},
                        {"term": {"response": 200}},
                        {"term": {"verb": "GET"}}
                    ],
                    "must_not": [
                        {"term": {"auth": "-"}},
                        {"wildcard": {"request": "*.xml"}},
                        {"wildcard": {"request": "*QUICKLOOK*"}},
                        {"wildcard": {"clientip": "192.168.*"}} # additional filtering on client IP-addresses
                    ]
                }
            }
        }

        # additional filtering on date
        if start is not None and end is not None:
            LOGGER.debug(f'Adding time filter: [{start} TO {end}]')
            from datetime import datetime
            query['query']['bool']['must'].append({"range": {"@timestamp": {"gt": StatsExporter._datetime2str(start),
                                                                            "lt": StatsExporter._datetime2str(end)}}})
        elif start is not None:
            LOGGER.debug(f'Adding time filter: [{start} TO *]')
            query['query']['bool']['must'].append({"range": {"@timestamp": {"gt": start.isoformat()}}})
        elif end is not None:
            LOGGER.debug(f'Adding time filter: [* TO {end}]')
            query['query']['bool']['must'].append({"range": {"@timestamp": {"lt": StatsExporter._datetime2str(end)}}})

        response = es.search(
            index="filebeat-index-6m*",
            body=query,
            scroll=es_scroll_timeout,
            size=100
        )

        count_hits = 0

        while response['hits']['hits']:
            es_scroll_id = response['_scroll_id']
            hits = response['hits']['hits']

            count_hits += len(hits)

            stats = []
            for hit in hits:
                LOGGER.debug(f'Converting hit {hit} to stat object')
                stat = Stat.from_es_record(hit)
                stats.append(stat)

            exporter.export(stats)

            response = es.scroll(scroll_id=es_scroll_id, scroll=es_scroll_timeout)

        LOGGER.info(f'Exported {count_hits} download statistics')

==> oscars_etl-0.1.5.20210223.16-py3-none-any.whl: oscars_etl/stats.py <==
import logging

LOGGER = logging.getLogger(__name__)


class Stat(object):

    def __init__(self, username, timestamp, collection,
                 product_id, product_date, product_size):

        self.username = username
        self.timestamp = timestamp
        self.collection = collection
        self.product_id = product_id
        self.product_date = product_date
        self.product_size = product_size

    @staticmethod
    def from_es_record(es_record):
        request = es_record['_source']['request']
        return Stat(es_record['_source']['auth'],
                    Stat._str_to_datetime(es_record['_source']['@timestamp']),
                    Stat._get_collection(request),
                    Stat._get_product_id(request),
                    Stat._get_product_date(request),
                    es_record['_source']['bytes'])

    @staticmethod
    def _clean_request(request):
        import re
        # remove download prefix
        request = re.sub('.*download/{1,}', '', request)
        # replace 2 or more slashes by one
        return re.sub('/{2,}', '/', request)

    @staticmethod
    def _str_to_datetime(timestamp):
        import dateutil.parser
        return dateutil.parser.isoparse(timestamp)

    @staticmethod
    def _get_collection(request):
        request = Stat._clean_request(request)
        url_parts = request.split('/')
        if url_parts[0] in ['Sentinel2', 'Sentinel5P', 'PROBAV']:
            return url_parts[0] + '_' + url_parts[1]
        else:
            return url_parts[0]

    @staticmethod
    def _get_product_id(request):
        collection = Stat._get_collection(request)
        request = Stat._clean_request(request)
        import re
        if re.match('Sentinel2_*', collection):
            url_parts = request.split('/')
            return url_parts[5]
        elif re.match('CGS_S1_SLC_L1', collection):
            url_parts = request.split('/')
            return url_parts[6]
        elif re.match('CGS_S1_GRD_L1', collection):
            url_parts = request.split('/')
            return url_parts[7]
        elif re.match('PROBAV_*', collection) \
                or re.match('CGS_S1_GRD_SIGMA0*', collection) \
                or re.match('S1_*', collection) \
                or re.match('CGS_S2_L1C', collection):
            url_parts = request.split('/')
            return url_parts[4]
        elif re.match('Sentinel5P_*', collection):
            url_parts = request.split('/')
            return url_parts[-2]
        else:
            raise Exception('Unsupported collection {}'.format(collection))

    @staticmethod
    def _get_product_date(request):
        collection = Stat._get_collection(request)
        request = Stat._clean_request(request)
        import re
        from datetime import datetime
        if re.match('Sentinel2_*', collection):
            url_parts = request.split('/')
            return datetime(int(url_parts[2]), int(url_parts[3]), int(url_parts[4]))
        elif re.match('CGS_S1_SLC_L1', collection):
            url_parts = request.split('/')
            return datetime(int(url_parts[3]), int(url_parts[4]), int(url_parts[5]))
        elif re.match('CGS_S1_GRD_L1', collection):
            url_parts = request.split('/')
            return datetime(int(url_parts[4]), int(url_parts[5]), int(url_parts[6]))
        elif re.match('S1_*', collection) \
                or re.match('CGS_S1_GRD_SIGMA0*', collection) \
                or re.match('CGS_S2_L1C', collection):
            url_parts = request.split('/')
            return datetime(int(url_parts[1]), int(url_parts[2]), int(url_parts[3]))
        elif re.match('PROBAV_*', collection):
            url_parts = request.split('/')
            return datetime.strptime(url_parts[3], '%Y%m%d')
        elif re.match('Sentinel5P_*', collection):
            product_id = Stat._get_product_id(request)
            id_parts = product_id.split('_')
            return datetime.strptime(id_parts[5], '%Y%m%d')
        else:
            raise Exception('Unsupported collection {}'.format(collection))


class CsvExporter(object):

    def __init__(self, csv_file):
        self.csv_file = csv_file

    def export(self, stats):
        with open(self.csv_file, 'a+') as stats_file:
            import csv
            stats_writer = csv.writer(stats_file, delimiter=',')
            for stat in stats:
                stats_writer.writerow([stat.username, stat.timestamp, stat.collection,
                                       stat.product_date, stat.product_id, stat.product_size])


class DatabaseExporter(object):

    def __init__(self, db_host, db_name, db_username, db_password):

        self.db_host = db_host
        self.db_name = db_name
        self.db_username = db_username
        self.db_password = db_password

        try:
            import psycopg2
            self.conn = psycopg2.connect("host='{}' dbname='{}' user='{}' password='{}'"
                                    .format(self.db_host, self.db_name, self.db_username, self.db_password))
        except Exception as ex:
            LOGGER.error('Could not connect to database: {}'.format(ex))
            raise

    def get_latest_stat(self):

        from psycopg2 import sql

        cursor = None
        try:
            cursor = self.conn.cursor()
            cursor.execute(
                sql.SQL("select max(download_timestamp) from oscars_stats"))
            results = cursor.fetchone()
            return results[0]
        finally:
            if cursor is not None:
                cursor.close()

    def export(self, stats):

        from psycopg2 import sql

        cursor = None
        try:
            cursor = self.conn.cursor()
            for stat in stats:
                cursor.execute(
                    sql.SQL(
                        """INSERT INTO oscars_stats (username, download_timestamp, collection, product_date, product_id, product_size) 
                            VALUES (%s, %s, %s, %s, %s, %s)"""),
                    [stat.username, stat.timestamp, stat.collection, stat.product_date, stat.product_id, stat.product_size])

            self.conn.commit()
        finally:
            if cursor is not None:
                cursor.close()

    def close(self):
        if self.conn is not None:
            self.conn.close()


class StatsExporter(object):

    def __init__(self, es_hosts):
        self.es_hosts = es_hosts

    def export_csv(self, start, end, csv_file):

        exporter = CsvExporter(csv_file)
        self._export_stats(start, end, exporter)

    def export_db(self, start, end,
                  db_host, db_name, db_username, db_password):

        exporter = DatabaseExporter(db_host, db_name, db_username, db_password)

        if start == None:
            # try to retrieve most recent download from DB
            start = exporter.get_latest_stat()

        self._export_stats(start, end, exporter)

    @staticmethod
    def _datetime2str(timestamp):
        import dateutil.parser
        return timestamp.strftime('%Y-%m-%dT%H:%M:%S.000Z')

    def _export_stats(self, start, end, exporter):

        from elasticsearch7 import Elasticsearch
        es = Elasticsearch(hosts=self.es_hosts)

        es_scroll_timeout = "1m"

        LOGGER.info("Start searching download statistics")

        query = {
            "query": {
                "bool": {
                    "must": [
                        {"wildcard": {"host.name": "oscars-download*.vgt.vito.be"}},
                        {"term": {"response": 200}},
                        {"term": {"verb": "GET"}}
                    ],
                    "must_not": [
                        {"term": {"auth": "-"}},
                        {"wildcard": {"request": "*.xml"}},
                        {"wildcard": {"request": "*QUICKLOOK*"}},
                        {"wildcard": {"clientip": "192.168.*"}} # additional filtering on client IP-addresses
                    ]
                }
            }
        }

        # additional filtering on date
        if start is not None and end is not None:
            LOGGER.debug(f'Adding time filter: [{start} TO {end}]')
            from datetime import datetime
            query['query']['bool']['must'].append({"range": {"@timestamp": {"gt": StatsExporter._datetime2str(start),
                                                                            "lt": StatsExporter._datetime2str(end)}}})
        elif start is not None:
            LOGGER.debug(f'Adding time filter: [{start} TO *]')
            query['query']['bool']['must'].append({"range": {"@timestamp": {"gt": start.isoformat()}}})
        elif end is not None:
            LOGGER.debug(f'Adding time filter: [* TO {end}]')
            query['query']['bool']['must'].append({"range": {"@timestamp": {"lt": StatsExporter._datetime2str(end)}}})

        response = es.search(
            index="filebeat-index-6m*",
            body=query,
            scroll=es_scroll_timeout,
            size=100
        )

        count_hits = 0

        while response['hits']['hits']:
            es_scroll_id = response['_scroll_id']
            hits = response['hits']['hits']

            count_hits += len(hits)

            stats = []
            for hit in hits:
                LOGGER.debug(f'Converting hit {hit} to stat object')
                stat = Stat.from_es_record(hit)
                stats.append(stat)

            exporter.export(stats)

            response = es.scroll(scroll_id=es_scroll_id, scroll=es_scroll_timeout)

        LOGGER.info(f'Exported {count_hits} download statistics')
